#!/usr/bin/env python
#####################################################################################
# Copyright (c) Members of the EGEE Collaboration. 2004 
# See http://eu-egee.org/partners/ for details on the copyright holders
# For license conditions see the license file or http://eu-egee.org/license.html
#####################################################################################
# glite-io-client-config v. 2.2.0
#
# Post-installation script for configuring the gLite I/O Client
# Alberto Di Meglio  < alberto.di.meglio@cern.ch >
# Leanne Guy  < leanne.guy@cern.ch >
# Joachim Flammer <Joachim.Flammer@cern.ch>
#
# Version info: $Id$
#
# Usage: python glite-io-client-config [OPTION]
#        -c, --checkconf        print configuration
#        -v, --version          print version
#        -h,--help              print usage info
#        --configure            configure the I/O Client
#        --siteconfig=URL  	    specify URL of the site configuration file
#		 --subservice           run as subservice script
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

import os,string
from shutil import copy
import sys
import getopt
import time

from  gLiteInstallerLib import gLib
from  gLiteInstallerLib import ConfigParams
from serviceDiscovery import ServiceDiscovery

# Set global variables here 
global params

class glite_io_client:
    
    def __init__(self):
        self.verbose = 0
        self.version = "2.2.0"
        self.name = "glite-io-client"
        self.friendly_name = "gLite I/O Client"
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "            Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
     
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --configure         configure the I/O Client'
        print '    --subservice        run script as a subservice'        
        print ''
        print '    --siteconfig=URL    URL of the site configuration file'
        print '\n'

    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------

    def configure(self):

		# subservice					
        if params.has_key('io.client.run_as_subservice'):
            subservice = params['io.client.run_as_subservice']
        else:
            subservice = "false"

        # ------------------------------------------------------------
        # configure service discovery
        # ------------------------------------------------------------
        print
        errorcode = serviceDiscovery.configure(params,glib)
        if errorcode:
            return 1            
            
        return 0
    
#-------------------------------------------------------------------------------
# Set environment variables
#-------------------------------------------------------------------------------

def set_env():

    if not os.geteuid() == 0:
        
        params['installer.export.filename'] = glib.checkSetEnvFile(params['installer.export.filename'])
        glib.printInfoMessage("gLite environment file will be saved to %s" % params['installer.export.filename'])
        
        installPrefix = os.path.abspath(glib.getScriptPath() + "/../../../")
        
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/lib' % installPrefix)
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/local/lib' % installPrefix)
        
        glib.addEnvPath('PATH','%s/usr/bin' % installPrefix)
        
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/site_perl/5.6.0/" % installPrefix)
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/vendor_perl/5.8.0/" % installPrefix)
        
        glib.printWarningMessage("The configuration script can modify the environment variables")
        print "    Modifications are listed below\n"
        
        if os.path.exists(params['GLITE_LOCATION']):
            if not os.access(params['GLITE_LOCATION'], os.W_OK):
                print "    GLITE_LOCATION is set to the non-writable directory %s -" % params['GLITE_LOCATION'],
                params['GLITE_LOCATION'] = "%s" % (installPrefix)
                print "it will be changed to %s" % params['GLITE_LOCATION']
            else:
                if not params['GLITE_LOCATION'] == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION'] = "%s" % (installPrefix)

        if not os.path.exists(params['GLOBUS_LOCATION']):
            print "    GLOBUS_LOCATION is set to the non-existing directory %s " % (params['GLOBUS_LOCATION']),
            params['GLOBUS_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GLOBUS_LOCATION']

        if not os.path.exists(params['GPT_LOCATION']):
            print "    GPT_LOCATION is set to the non-existing directory %s " % (params['GPT_LOCATION']),
            params['GPT_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GPT_LOCATION']

        if os.path.exists(params['GLITE_LOCATION_VAR']):
            if not os.access('GLITE_LOCATION_VAR', os.W_OK):
                print "    GLITE_LOCATION_VAR is set to the non-writable directory %s - " % params['GLITE_LOCATION_VAR'],
                params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
                print "value changed to %s" % params['GLITE_LOCATION_VAR']
            else:
                if not params['GLITE_LOCATION_VAR'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
        
        if os.path.exists(params['GLITE_LOCATION_LOG']):
            if not os.access('GLITE_LOCATION_LOG', os.W_OK):
                print "    GLITE_LOCATION_LOG is set to the non-writable directory %s - " % params['GLITE_LOCATION_LOG'],
                params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
                print "it will be changed to %s" % params['GLITE_LOCATION_LOG']
            else:
                if not params['GLITE_LOCATION_LOG'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
        
        if os.path.exists(params['ca.certificates.dir']):
            if not os.access('ca.certificates.dir', os.W_OK):
                print "    ca.certificates.dir is set to the non-writable directory %s - " % params['ca.certificates.dir'],
                params['ca.certificates.dir'] = "%s/etc/grid-security/certificates" % installPrefix
                print "it will be changed to %s" % params['ca.certificates.dir']
            else:
                if not params['ca.certificates.dir'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['ca.certificates.dir'] = "%s/etc/grid-security/certificates" % installPrefix
        
        print "\n"

    # Standard configuration
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    glib.export('GLITE_LOCATION_LOG');
    glib.export('GLITE_LOCATION_TMP');
    glib.check_dir(os.environ['GLITE_LOCATION_VAR'])
    glib.check_dir(os.environ['GLITE_LOCATION_LOG'])
    glib.check_dir(os.environ['GLITE_LOCATION_TMP'])

    # Globus settings
    glib.export('GLOBUS_LOCATION');

    # bin and lib paths
    glib.addEnvPath("PATH","$GLOBUS_LOCATION/bin")
    glib.addEnvPath("PATH","$GLITE_LOCATION/externals/bin")
    glib.addEnvPath("PATH","$GLITE_LOCATION/bin")

    glib.addEnvPath("LD_LIBRARY_PATH","$GLOBUS_LOCATION/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","$GLITE_LOCATION/externals/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","$GLITE_LOCATION/lib")

    # Set environment
    glib.setUserEnv()
    
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = gLib()
    
    # ------------------------------------------------------------
    # Load parameters
    # ------------------------------------------------------------
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
        
    configFiles = ["%s/../glite-io-client.cfg.xml" % glib.getScriptPath(), 
                   "%s/../glite-service-discovery.cfg.xml" % glib.getScriptPath()]
        
    if glib.loadConfiguration(configFiles,params):
        print "An error occurred while configuring the service"
        sys.exit(1)    
    
    # ------------------------------------------------------------    
    # set verbosity
    # ------------------------------------------------------------    
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    glib.verbose = verbose
    
    # Set up the environment
    set_env()
          
    # ------------------------------------------------------------          
    # Instantiate the service classes
    # ------------------------------------------------------------
    service = glite_io_client()
    service.verbose = verbose

    serviceDiscovery = ServiceDiscovery()
    serviceDiscovery.verbose = verbose
     
    # ------------------------------------------------------------    
    # Command line opts if any
    # ------------------------------------------------------------    
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'configure', 'siteconfig=', 'subservice'])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    if len(opts) == 0:
        service.usage()
        sys.exit(0)
    
    # ------------------------------------------------------------
    # Check cli options
    # ------------------------------------------------------------

    configure=0
    subservice=0

    for o, a in opts:
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
            
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
            
        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)

        # configure service as subservice
        if o == "--subservice":
            subservice = 1
            params['io.client.run_as_subservice'] = "true"			

        # configure
        if o == "--configure":
            configure=1

    #-------------------------------------------------------------------------------    
    # Configure
    #-------------------------------------------------------------------------------	
    if configure==1:
		
        #-------------------------------------------------------------------------------    
        # Print configuration parameters
        #-------------------------------------------------------------------------------	
        if verbose:
            glib.print_params(params)

        #-------------------------------------------------------------------------------
        # print default statements
        #-------------------------------------------------------------------------------	
        if subservice==0:
            service.copyright()
            service.showVersion()
	   
        # print banner
        service.banner()
	
        #-------------------------------------------------------------------------------
        # Configure the IO client
        #-------------------------------------------------------------------------------	
        if service.configure() == 0:
            print "\n\n%s configuration successfully completed\n" % service.friendly_name
        else:
            print "\n\nAn unrecoverable error occurred while configuring the %s" \
                % service.friendly_name
            sys.exit(1)

        sys.exit(0)
            
    else:
        # ------------------------------------------------------------
        # No option given
        # ------------------------------------------------------------
        service.usage()
        sys.exit(0)			
            